# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mir_common.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='mir_common.proto',
  package='ymir.common',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x10mir_common.proto\x12\x0bymir.common*Z\n\x07TvtType\x12\x12\n\x0eTvtTypeUnknown\x10\x00\x12\x13\n\x0fTvtTypeTraining\x10\x01\x12\x15\n\x11TvtTypeValidation\x10\x02\x12\x0f\n\x0bTvtTypeTest\x10\x03*\xf1\x01\n\tAssetType\x12\x14\n\x10\x41ssetTypeUnknown\x10\x00\x12\x16\n\x12\x41ssetTypeImageJpeg\x10\x01\x12\x15\n\x11\x41ssetTypeImagePng\x10\x02\x12\x1a\n\x16\x41ssetTypeImagePixelMat\x10\x03\x12\x19\n\x15\x41ssetTypeImageYuv420p\x10\x04\x12\x1a\n\x16\x41ssetTypeImageYuv420sp\x10\x05\x12\x19\n\x15\x41ssetTypeImageYuv422p\x10\x06\x12\x1a\n\x16\x41ssetTypeImageYuv422sp\x10\x07\x12\x15\n\x11\x41ssetTypeVideoMp4\x10\x65*\xc9\x01\n\x08TaskType\x12\x13\n\x0fTaskTypeUnknown\x10\x00\x12\x14\n\x10TaskTypeTraining\x10\x01\x12\x12\n\x0eTaskTypeMining\x10\x02\x12\x11\n\rTaskTypeLabel\x10\x03\x12\x12\n\x0eTaskTypeFilter\x10\x04\x12\x16\n\x12TaskTypeImportData\x10\x05\x12\x16\n\x12TaskTypeExportData\x10\x06\x12\x14\n\x10TaskTypeCopyData\x10\x07\x12\x11\n\rTaskTypeMerge\x10\x08*\x87\x01\n\tTaskState\x12\x14\n\x10TaskStateUnknown\x10\x00\x12\x14\n\x10TaskStatePending\x10\x01\x12\x14\n\x10TaskStateRunning\x10\x02\x12\x11\n\rTaskStateDone\x10\x03\x12\x12\n\x0eTaskStateError\x10\x04\x12\x11\n\rTaskStateMiss\x10\x05*L\n\x08Sha1Type\x12\x15\n\x11SHA1_TYPE_UNKNOWN\x10\x00\x12\x13\n\x0fSHA1_TYPE_ASSET\x10\x01\x12\x14\n\x10SHA1_TYPE_COMMIT\x10\x02*\xf6\x04\n\x05RCode\x12\t\n\x05RC_OK\x10\x00\x12\x14\n\x10RC_ERROR_UNKNOWN\x10\x01\x12\x18\n\x14RC_CMD_ERROR_UNKNOWN\x10\x64\x12\x1b\n\x17RC_CMD_CORRUPT_MIR_FILE\x10\x65\x12\x1b\n\x17RC_CMD_INVALID_MIR_FILE\x10\x66\x12\x1b\n\x17RC_CMD_INVALID_MIR_REPO\x10g\x12\x17\n\x13RC_CMD_INVALID_ARGS\x10h\x12 \n\x1cRC_CMD_INVALID_BRANCH_OR_TAG\x10i\x12\x1a\n\x16RC_CMD_INVALID_COMMAND\x10j\x12\x1d\n\x19RC_CMD_MIR_FILE_NOT_FOUND\x10k\x12\x1c\n\x18RC_CMD_CONFLICTS_OCCURED\x10l\x12\x1a\n\x16RC_CMD_EMPTY_METADATAS\x10m\x12\x1a\n\x16RC_CMD_EMPTY_TRAIN_SET\x10n\x12\x18\n\x14RC_CMD_EMPTY_VAL_SET\x10o\x12\x15\n\x11RC_CMD_DIRTY_REPO\x10p\x12\x1b\n\x17RC_CMD_NOTHING_TO_MERGE\x10q\x12\x1d\n\x18RC_SERVICE_ERROR_UNKNOWN\x10\xe8\x07\x12\x1b\n\x16RC_SERVICE_INVALID_REQ\x10\xe9\x07\x12\x1c\n\x17RC_SERVICE_INVOKE_ERROR\x10\xea\x07\x12\x1f\n\x1aRC_SERVICE_UNKOWN_RESPONSE\x10\xeb\x07\x12\"\n\x1dRC_SERVICE_TASK_INVOKER_ERROR\x10\xec\x07\x12\"\n\x1dRC_SERVICE_INFO_INVOKER_ERROR\x10\xed\x07*U\n\nMirStorage\x12\x11\n\rMIR_METADATAS\x10\x00\x12\x13\n\x0fMIR_ANNOTATIONS\x10\x01\x12\x10\n\x0cMIR_KEYWORDS\x10\x02\x12\r\n\tMIR_TASKS\x10\x03*<\n\x0bLabelFormat\x12\x11\n\rNO_ANNOTATION\x10\x00\x12\x0e\n\nPASCAL_VOC\x10\x01\x12\n\n\x06IF_ARK\x10\x02\x62\x06proto3'
)

_TVTTYPE = _descriptor.EnumDescriptor(
  name='TvtType',
  full_name='ymir.common.TvtType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TvtTypeUnknown', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TvtTypeTraining', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TvtTypeValidation', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TvtTypeTest', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=33,
  serialized_end=123,
)
_sym_db.RegisterEnumDescriptor(_TVTTYPE)

TvtType = enum_type_wrapper.EnumTypeWrapper(_TVTTYPE)
_ASSETTYPE = _descriptor.EnumDescriptor(
  name='AssetType',
  full_name='ymir.common.AssetType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='AssetTypeUnknown', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AssetTypeImageJpeg', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AssetTypeImagePng', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AssetTypeImagePixelMat', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AssetTypeImageYuv420p', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AssetTypeImageYuv420sp', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AssetTypeImageYuv422p', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AssetTypeImageYuv422sp', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AssetTypeVideoMp4', index=8, number=101,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=126,
  serialized_end=367,
)
_sym_db.RegisterEnumDescriptor(_ASSETTYPE)

AssetType = enum_type_wrapper.EnumTypeWrapper(_ASSETTYPE)
_TASKTYPE = _descriptor.EnumDescriptor(
  name='TaskType',
  full_name='ymir.common.TaskType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TaskTypeUnknown', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TaskTypeTraining', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TaskTypeMining', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TaskTypeLabel', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TaskTypeFilter', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TaskTypeImportData', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TaskTypeExportData', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TaskTypeCopyData', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TaskTypeMerge', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=370,
  serialized_end=571,
)
_sym_db.RegisterEnumDescriptor(_TASKTYPE)

TaskType = enum_type_wrapper.EnumTypeWrapper(_TASKTYPE)
_TASKSTATE = _descriptor.EnumDescriptor(
  name='TaskState',
  full_name='ymir.common.TaskState',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TaskStateUnknown', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TaskStatePending', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TaskStateRunning', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TaskStateDone', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TaskStateError', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TaskStateMiss', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=574,
  serialized_end=709,
)
_sym_db.RegisterEnumDescriptor(_TASKSTATE)

TaskState = enum_type_wrapper.EnumTypeWrapper(_TASKSTATE)
_SHA1TYPE = _descriptor.EnumDescriptor(
  name='Sha1Type',
  full_name='ymir.common.Sha1Type',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SHA1_TYPE_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SHA1_TYPE_ASSET', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SHA1_TYPE_COMMIT', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=711,
  serialized_end=787,
)
_sym_db.RegisterEnumDescriptor(_SHA1TYPE)

Sha1Type = enum_type_wrapper.EnumTypeWrapper(_SHA1TYPE)
_RCODE = _descriptor.EnumDescriptor(
  name='RCode',
  full_name='ymir.common.RCode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RC_OK', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_ERROR_UNKNOWN', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_ERROR_UNKNOWN', index=2, number=100,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_CORRUPT_MIR_FILE', index=3, number=101,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_INVALID_MIR_FILE', index=4, number=102,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_INVALID_MIR_REPO', index=5, number=103,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_INVALID_ARGS', index=6, number=104,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_INVALID_BRANCH_OR_TAG', index=7, number=105,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_INVALID_COMMAND', index=8, number=106,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_MIR_FILE_NOT_FOUND', index=9, number=107,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_CONFLICTS_OCCURED', index=10, number=108,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_EMPTY_METADATAS', index=11, number=109,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_EMPTY_TRAIN_SET', index=12, number=110,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_EMPTY_VAL_SET', index=13, number=111,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_DIRTY_REPO', index=14, number=112,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_CMD_NOTHING_TO_MERGE', index=15, number=113,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_SERVICE_ERROR_UNKNOWN', index=16, number=1000,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_SERVICE_INVALID_REQ', index=17, number=1001,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_SERVICE_INVOKE_ERROR', index=18, number=1002,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_SERVICE_UNKOWN_RESPONSE', index=19, number=1003,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_SERVICE_TASK_INVOKER_ERROR', index=20, number=1004,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RC_SERVICE_INFO_INVOKER_ERROR', index=21, number=1005,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=790,
  serialized_end=1420,
)
_sym_db.RegisterEnumDescriptor(_RCODE)

RCode = enum_type_wrapper.EnumTypeWrapper(_RCODE)
_MIRSTORAGE = _descriptor.EnumDescriptor(
  name='MirStorage',
  full_name='ymir.common.MirStorage',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='MIR_METADATAS', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MIR_ANNOTATIONS', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MIR_KEYWORDS', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MIR_TASKS', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1422,
  serialized_end=1507,
)
_sym_db.RegisterEnumDescriptor(_MIRSTORAGE)

MirStorage = enum_type_wrapper.EnumTypeWrapper(_MIRSTORAGE)
_LABELFORMAT = _descriptor.EnumDescriptor(
  name='LabelFormat',
  full_name='ymir.common.LabelFormat',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NO_ANNOTATION', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PASCAL_VOC', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='IF_ARK', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1509,
  serialized_end=1569,
)
_sym_db.RegisterEnumDescriptor(_LABELFORMAT)

LabelFormat = enum_type_wrapper.EnumTypeWrapper(_LABELFORMAT)
TvtTypeUnknown = 0
TvtTypeTraining = 1
TvtTypeValidation = 2
TvtTypeTest = 3
AssetTypeUnknown = 0
AssetTypeImageJpeg = 1
AssetTypeImagePng = 2
AssetTypeImagePixelMat = 3
AssetTypeImageYuv420p = 4
AssetTypeImageYuv420sp = 5
AssetTypeImageYuv422p = 6
AssetTypeImageYuv422sp = 7
AssetTypeVideoMp4 = 101
TaskTypeUnknown = 0
TaskTypeTraining = 1
TaskTypeMining = 2
TaskTypeLabel = 3
TaskTypeFilter = 4
TaskTypeImportData = 5
TaskTypeExportData = 6
TaskTypeCopyData = 7
TaskTypeMerge = 8
TaskStateUnknown = 0
TaskStatePending = 1
TaskStateRunning = 2
TaskStateDone = 3
TaskStateError = 4
TaskStateMiss = 5
SHA1_TYPE_UNKNOWN = 0
SHA1_TYPE_ASSET = 1
SHA1_TYPE_COMMIT = 2
RC_OK = 0
RC_ERROR_UNKNOWN = 1
RC_CMD_ERROR_UNKNOWN = 100
RC_CMD_CORRUPT_MIR_FILE = 101
RC_CMD_INVALID_MIR_FILE = 102
RC_CMD_INVALID_MIR_REPO = 103
RC_CMD_INVALID_ARGS = 104
RC_CMD_INVALID_BRANCH_OR_TAG = 105
RC_CMD_INVALID_COMMAND = 106
RC_CMD_MIR_FILE_NOT_FOUND = 107
RC_CMD_CONFLICTS_OCCURED = 108
RC_CMD_EMPTY_METADATAS = 109
RC_CMD_EMPTY_TRAIN_SET = 110
RC_CMD_EMPTY_VAL_SET = 111
RC_CMD_DIRTY_REPO = 112
RC_CMD_NOTHING_TO_MERGE = 113
RC_SERVICE_ERROR_UNKNOWN = 1000
RC_SERVICE_INVALID_REQ = 1001
RC_SERVICE_INVOKE_ERROR = 1002
RC_SERVICE_UNKOWN_RESPONSE = 1003
RC_SERVICE_TASK_INVOKER_ERROR = 1004
RC_SERVICE_INFO_INVOKER_ERROR = 1005
MIR_METADATAS = 0
MIR_ANNOTATIONS = 1
MIR_KEYWORDS = 2
MIR_TASKS = 3
NO_ANNOTATION = 0
PASCAL_VOC = 1
IF_ARK = 2


DESCRIPTOR.enum_types_by_name['TvtType'] = _TVTTYPE
DESCRIPTOR.enum_types_by_name['AssetType'] = _ASSETTYPE
DESCRIPTOR.enum_types_by_name['TaskType'] = _TASKTYPE
DESCRIPTOR.enum_types_by_name['TaskState'] = _TASKSTATE
DESCRIPTOR.enum_types_by_name['Sha1Type'] = _SHA1TYPE
DESCRIPTOR.enum_types_by_name['RCode'] = _RCODE
DESCRIPTOR.enum_types_by_name['MirStorage'] = _MIRSTORAGE
DESCRIPTOR.enum_types_by_name['LabelFormat'] = _LABELFORMAT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)


# @@protoc_insertion_point(module_scope)



from abc import ABC, abstractmethod
import logging


class BaseCommand(ABC):
    """base class for all commands in mir"""

    # lifecycle
    def __init__(self, args):
        self.args = args

    # public: abstract
    @abstractmethod
    def run(self):
        logging.critical("BaseCommand run: this action should override in sub classes")



import logging

from mir.cli import create_main_parser


def main(argv=None) -> int:
    """
    Run dvc CLI command.
    Args:
        argv: optional list of arguments to parse.
    Returns:
        int: command's return code.
    """

    # parse and exec
    argument_parser = create_main_parser()
    args = argument_parser.parse_args(argv)
    if hasattr(args, 'func'):
        cmd = args.func(args)  # see cli.py: bind_to_subparsers for all elements in _COMMANDS_
        return cmd.run()

    argument_parser.print_help()
    return -1



import struct

from dataclasses import dataclass
from enum import Enum, IntEnum, unique


class IDProto(IntEnum):
    ID_LEN_ID_TYPE = 1
    ID_LEN_SUBTASK_ID = 1
    ID_LEN_RESERVE = 2
    ID_LEN_USER_ID = 4
    ID_LEN_REPO_ID = 6
    ID_LEN_HEX_TASK_ID = 16
    ID_LENGTH = (ID_LEN_ID_TYPE + ID_LEN_SUBTASK_ID + ID_LEN_RESERVE + ID_LEN_USER_ID + ID_LEN_REPO_ID +
                 ID_LEN_HEX_TASK_ID)


@unique
class IDType(Enum):
    ID_TYPE_UNKNOWN = 'z'
    ID_TYPE_ASSET = 'a'
    ID_TYPE_COMMIT = 'c'
    ID_TYPE_TASK = 't'
    ID_TYPE_REPO = 'r'
    ID_TYPE_USER = 'u'


@dataclass
class TaskId:
    id_type: str
    sub_task_id: str
    id_reserve: str
    user_id: str
    repo_id: str
    hex_task_id: str

    def __post_init__(self):
        if len(self.id_type) != IDProto.ID_LEN_ID_TYPE:
            raise ValueError(f"Invalid id_type: {self.id_type}")
        if len(self.sub_task_id) != IDProto.ID_LEN_SUBTASK_ID:
            raise ValueError(f"Invalid sub_task_id: {self.sub_task_id}")
        if len(self.id_reserve) != IDProto.ID_LEN_RESERVE:
            raise ValueError(f"Invalid id_reserve: {self.id_reserve}")
        if len(self.user_id) != IDProto.ID_LEN_USER_ID:
            raise ValueError(f"Invalid user_id: {self.user_id}")
        if len(self.repo_id) != IDProto.ID_LEN_REPO_ID:
            raise ValueError(f"Invalid repo_id: {self.repo_id}")
        if len(self.hex_task_id) != IDProto.ID_LEN_HEX_TASK_ID:
            raise ValueError(f"Invalid hex_task_id: {self.hex_task_id}")

    def __str__(self):
        return f"{self.id_type}{self.sub_task_id}{self.id_reserve}{self.user_id}{self.repo_id}{self.hex_task_id}"

    @classmethod
    def from_task_id(cls, task_id: str):
        fmt = '1s1s2s4s6s16s'
        components = struct.unpack(fmt, task_id.encode())
        return cls(*(c.decode() for c in components))



import logging
import os
import pathlib
import shutil
import subprocess
import tarfile
from typing import List
import time
import yaml

from mir.commands import base
from mir.tools import checker, code, data_exporter, mir_storage, mir_storage_ops, revs_parser
from mir.tools import utils as mir_utils
from ymir_proto.gen import mir_common_pb2 as mir_common
from ymir_proto.gen import mir_entities_pb2 as mirpb


class CmdMining(base.BaseCommand):
    def run(self):
        logging.debug("command mine: %s", self.args)

        return CmdMining.run_with_args(export_root=self.args.work_dir,
                                       src_revs=self.args.src_revs,
                                       dst_rev=self.args.dst_rev,
                                       mir_root=self.args.mir_root,
                                       model_hash=self.args.model_hash,
                                       model_location=self.args.model_location,
                                       media_location=self.args.media_location,
                                       topk=self.args.topk)

    @staticmethod
    def run_with_args(export_root, src_revs, dst_rev, mir_root, model_hash, media_location, model_location, topk=0):
        """
        runs a mining task \n
        Args:
            export_root: mining docker container's work directory
            src_revs: data branch name and base task id
            dst_rev: destination branch name and task id
            mir_root: mir repo path, in order to run in non-mir folder.
            model_hash: used to target model, use prep_tid if non-set
            media_location, model_location: location of assets.
        Returns:
            error code
        """
        # Step 1: check and prepare environment.
        if not mir_root:
            mir_root = '.'
        if not export_root:
            logging.error("empty export_root: abort")
            return code.MirCode.RC_ERROR_INVALID_ARGS
        if not media_location or not model_location:
            logging.error("media or model location cannot be none!")
            return code.MirCode.RC_ERROR_INVALID_ARGS
        if not model_hash:
            logging.error("model_hash is required.")
            return code.MirCode.RC_ERROR_INVALID_ARGS

        if not src_revs or not dst_rev:
            logging.error("invalid --src-revs or --dst-rev; abort")
            return code.MirCode.RC_ERROR_INVALID_ARGS
        src_typ_rev_tid = revs_parser.parse_single_arg_rev(src_revs)
        if not src_typ_rev_tid.rev:
            logging.error(f"no rev found in --src-revs: {src_revs}")
            return code.MirCode.RC_ERROR_INVALID_ARGS
        dst_typ_rev_tid = revs_parser.parse_single_arg_rev(dst_rev)
        if not dst_typ_rev_tid.rev or not dst_typ_rev_tid.tid:
            logging.error(f"no rev or tid found in --dst-rev: {dst_rev}")
            return code.MirCode.RC_ERROR_INVALID_ARGS

        return_code = checker.check(mir_root)
        if return_code != code.MirCode.RC_OK:
            return return_code

        export_in = os.path.join(export_root, "in")  # docker container's input data directory
        export_out = os.path.join(export_root, "out")  # docker container's output data directory
        ret = _prepare_env(export_root=export_root, export_in=export_in, export_out=export_out)
        if ret != code.MirCode.RC_OK:
            logging.error("envron error!")
            return ret

        # Step 3: place data in required format and location.
        _parepare_mining_data(mir_root=mir_root,
                              base_branch=src_typ_rev_tid.rev,
                              input_path=export_in,
                              model_hash=model_hash,
                              model_location=model_location,
                              media_location=media_location,
                              task_id=dst_typ_rev_tid.tid)

        # Step 4: Mining job, start mining docker and wait.
        _run_docker_command(v_in=export_in, v_out=export_out, docker_image='al:0.0.1.b')

        # Step 4: Update mir commits.
        _process_mining_results(mir_root=mir_root,
                                export_out=export_out,
                                dst_typ_rev_tid=dst_typ_rev_tid,
                                src_typ_rev_tid=src_typ_rev_tid,
                                model_hash=model_hash,
                                topk=topk)
        logging.info("mining done, results at: %s", export_out)

        return code.MirCode.RC_OK


def _process_mining_results(mir_root: str, export_out: str, dst_typ_rev_tid: revs_parser.TypRevTid,
                            src_typ_rev_tid: revs_parser.TypRevTid, model_hash: str, topk: int):
    # step 1: build topk results:
    if topk < 0:
        raise RuntimeError("topk cannot be negative")
    topk_result_tsv = os.path.join(export_out, 'result.tsv')
    if not os.path.isfile(topk_result_tsv):
        raise RuntimeError("Cannot find result file {}".format(topk_result_tsv))
    idx_cnt, ret_asset_set = 0, set()
    with open(topk_result_tsv) as ret_f:
        for line in ret_f:
            line = line.strip()
            if not line:
                continue
            if topk > 0 and idx_cnt >= topk:
                break
            ret_asset_set.add(os.path.basename(line.split('\t')[0]))
            idx_cnt += 1
    logging.info("top {} samples found".format(len(ret_asset_set)))

    # step 2: update mir data files
    mir_datas = mir_storage_ops.MirStorageOps.load(mir_root=mir_root,
                                                   mir_branch=src_typ_rev_tid.rev,
                                                   mir_storages=mir_storage.get_all_mir_storage())
    mir_metadatas = mir_datas[mir_common.MirStorage.MIR_METADATAS]
    mir_annotations = mir_datas[mir_common.MirStorage.MIR_ANNOTATIONS]
    mir_keywords = mir_datas[mir_common.MirStorage.MIR_KEYWORDS]
    mir_tasks = mir_datas[mir_common.MirStorage.MIR_TASKS]

    # update mir metadatas
    matched_mir_metadatas = mirpb.MirMetadatas()
    for asset_id in ret_asset_set:
        matched_mir_metadatas.attributes[asset_id].CopyFrom(mir_metadatas.attributes[asset_id])
    logging.info("matched: %d, overriding metadatas.mir", len(matched_mir_metadatas.attributes))

    # update mir annotations
    task_annotation = mir_annotations.task_annotations[mir_annotations.head_task_id]
    joint_asset_ids_set = set(task_annotation.image_annotations.keys()) & ret_asset_set
    matched_mir_annotations = mirpb.MirAnnotations()
    matched_task_annotation = matched_mir_annotations.task_annotations[dst_typ_rev_tid.tid]
    for asset_id in joint_asset_ids_set:
        matched_task_annotation.image_annotations[asset_id].CopyFrom(task_annotation.image_annotations[asset_id])

    # update mir keywords
    joint_asset_ids_set = set(mir_keywords.keywords.keys()) & ret_asset_set
    matched_mir_keywords = mirpb.MirKeywords()
    for asset_id in joint_asset_ids_set:
        matched_mir_keywords.keywords[asset_id].CopyFrom(mir_keywords.keywords[asset_id])

    # update_mir_task
    task = mirpb.Task()
    task.type = mir_common.TaskTypeMining
    task.name = "mining"
    task.task_id = dst_typ_rev_tid.tid
    task.timestamp = int(time.time())
    task.model.model_hash = model_hash
    mir_storage_ops.add_mir_task(mir_tasks, task)

    # step 3: store results and commit.
    # TODO: add inference results into annotations.
    mir_datas = {
        mir_common.MirStorage.MIR_METADATAS: matched_mir_metadatas,
        mir_common.MirStorage.MIR_ANNOTATIONS: matched_mir_annotations,
        mir_common.MirStorage.MIR_KEYWORDS: matched_mir_keywords,
        mir_common.MirStorage.MIR_TASKS: mir_tasks,
    }
    mir_storage_ops.MirStorageOps.save_and_commit(mir_root=mir_root,
                                                  his_branch=src_typ_rev_tid.rev,
                                                  mir_branch=dst_typ_rev_tid.rev,
                                                  mir_datas=mir_datas,
                                                  commit_message=dst_typ_rev_tid.tid)

    return code.MirCode.RC_OK


def _get_model_hash(mir_root: str, prep_tid: str):
    # read `mir_task: mirpb.Task` from current tasks.mir (with `prep_tid`)
    mir_pre_task = mirpb.MirTasks()
    mir_tasks_path = os.path.join(mir_root, "tasks.mir")
    if not os.path.isfile(mir_tasks_path):
        raise ValueError("tasks.mir not found")
    with open(mir_tasks_path, "rb") as f:
        mir_pre_task.ParseFromString(f.read())
    if prep_tid not in mir_pre_task.tasks.keys():
        raise ValueError("cannot find prep task id: {} in tasks.mir".format(prep_tid))
    model_hash = mir_pre_task.tasks[prep_tid].model_hash
    if not model_hash:
        raise ValueError("empty model hash for task: {}".format(prep_tid))
    return model_hash


def _prepare_env(export_root: str, export_in: str, export_out: str):
    if os.path.exists(export_root):
        shutil.rmtree(export_root)
    os.makedirs(export_root, exist_ok=True)
    os.makedirs(export_in, exist_ok=True)
    os.makedirs(export_out, exist_ok=True)

    return code.MirCode.RC_OK


def _run_docker_command(v_in: str, v_out: str, docker_image: str):
    if not os.path.isdir(v_in) or not os.path.isdir(v_out) or not docker_image:
        raise ValueError("run_docker_command: Invalid input args.")

    bind_path = "-v {}:/in -v {}:/out".format(v_in, v_out)
    cmd = "docker run -it --rm {} --user {}:{} {}".format(bind_path, os.getuid(), os.getgid(), docker_image)
    logging.info("starting mining docker container with cmd %s", cmd)
    run_result = subprocess.run(cmd.split(" "), check=True)  # run and wait

    # when mining process done
    if run_result.returncode != 0:
        logging.info("mining error occured: %s", run_result.returncode)
        return code.MirCode.RC_ERROR_UNKNOWN

    return run_result


def _unpack_models(tar_file: str, dest_root: str, sub_folder: str) -> list:
    if not os.path.isdir(dest_root) or not os.path.isfile(tar_file):
        raise ValueError("_unpack_models args error")
    dest_path = os.path.join(dest_root, sub_folder)
    os.makedirs(dest_path, exist_ok=True)

    params_file, json_file = None, None
    with tarfile.open(tar_file, 'r') as tar_gz:
        for item in tar_gz:
            logging.info("extracting %s, %s", item, dest_path)
            if 'json' in item.name:
                json_file = item.name
            if 'params' in item.name:
                params_file = item.name
            tar_gz.extract(item, dest_path)
    return [os.path.join(sub_folder, i) for i in [params_file, json_file]]


def _parepare_mining_data(mir_root: str, base_branch: str, input_path: str, model_hash: str, model_location: str,
                          media_location: str, task_id: str) -> None:

    # Step 1: prepare images.
    _place_media(mir_root=mir_root, base_branch=base_branch, input_path=input_path, media_location=media_location)

    # Step 2: place pretrained model.
    logging.info("fetching model hash {}".format(model_hash))
    [in_container_model_params_path, _] = _place_model(model_hash=model_hash,
                                                       root_path=input_path,
                                                       sub_folder='model',
                                                       model_location=model_location)

    # Step 3: update yaml config file.
    with open(str(pathlib.Path(__file__).parent.parent / "conf/mining-template.yaml")) as f:
        config = yaml.safe_load(f)
    config["model_params_path"] = os.path.join('/in', in_container_model_params_path)
    config["task_id"] = task_id
    logging.debug("config: {}".format(config))
    with open(os.path.join(input_path, 'config.yaml'), 'w') as f:
        yaml.dump(config, f)

    return code.MirCode.RC_OK


def _place_media(mir_root: str, base_branch: str, input_path: str, media_location: str):
    metadata = mir_storage_ops.MirStorageOps.load_single(mir_root=mir_root,
                                                         mir_branch=base_branch,
                                                         ms=mir_common.MirStorage.MIR_METADATAS)
    if len(metadata.attributes) == 0:
        raise ValueError("no assets found in metadatas.mir")

    candidate_path = os.path.join(input_path, 'candidate')
    os.makedirs(candidate_path, exist_ok=True)

    img_list = list(metadata.attributes.keys())  # type: List[str]
    data_exporter.export(mir_root=mir_root,
                         assets_location=media_location,
                         class_type_ids={},
                         asset_ids=img_list,
                         asset_dir=candidate_path,
                         annotation_dir=None,
                         need_suffix=True,
                         base_branch=base_branch,
                         format_type=data_exporter.ExportFormat.EXPORT_FORMAT_NO_ANNOTATION)
    data_exporter.generate_asset_index_file(asset_dir=candidate_path,
                                            rel_start_dir=candidate_path,
                                            index_file_path=os.path.join(candidate_path, 'index.tsv'))


def _place_model(model_hash: str, root_path: str, sub_folder: str, model_location: str):
    """
    place model (named by model_hash) from root_path to model_folder, under sub_folder folder.
    """
    model_id_rel_paths = mir_utils.store_assets_to_dir(asset_ids=[model_hash],
                                                       out_root=root_path,
                                                       sub_folder=sub_folder,
                                                       asset_location=model_location,
                                                       need_suffix=False)
    model_path_abs = os.path.join(root_path, model_id_rel_paths[model_hash])
    ret = _unpack_models(tar_file=model_path_abs, dest_root=root_path, sub_folder=sub_folder)
    return ret


def bind_to_subparsers(subparsers, parent_parser):  # pragma: no cover
    mining_arg_parser = subparsers.add_parser("mining",
                                              parents=[parent_parser],
                                              description="use this command to mine in current workspace",
                                              help="mine current workspace")
    mining_arg_parser.add_argument("-w",
                                   required=True,
                                   dest="work_dir",
                                   type=str,
                                   help="work place for mining and monitoring")
    mining_arg_parser.add_argument("--model-location",
                                   required=True,
                                   dest="model_location",
                                   type=str,
                                   help="model storage location for models")
    mining_arg_parser.add_argument("--media-location",
                                   required=True,
                                   dest="media_location",
                                   type=str,
                                   help="media storage location for models")
    mining_arg_parser.add_argument("--topk",
                                   dest="topk",
                                   type=int,
                                   default=0,
                                   help="If set, discard samples out of topk, sorting by scores.")
    mining_arg_parser.add_argument("--model-hash",
                                   dest="model_hash",
                                   type=str,
                                   required=True,
                                   help="model hash to be used")
    mining_arg_parser.add_argument("--src-revs",
                                   dest="src_revs",
                                   type=str,
                                   required=True,
                                   help="rev@bid: source rev and base task id")
    mining_arg_parser.add_argument("--dst-rev",
                                   dest="dst_rev",
                                   type=str,
                                   required=True,
                                   help="rev@tid: destination branch name and task id")
    mining_arg_parser.set_defaults(func=CmdMining)



import logging

from mir import scm
from mir.commands import base
from mir.tools import checker, code


class CmdLog(base.BaseCommand):
    def run_with_args(mir_root: str,
                      decorate: bool,
                      oneline: bool,
                      graph: bool,
                      dog: bool,
                      with_stdout: bool = False) -> int:
        return_code = checker.check(mir_root, [checker.Prerequisites.IS_INSIDE_MIR_REPO])
        if return_code != code.MirCode.RC_OK:
            return return_code

        cmd_opts = []
        if dog:
            decorate = True
            oneline = True
            graph = True
        if decorate:
            cmd_opts.append("--decorate")
        if oneline:
            cmd_opts.append("--oneline")
        if graph:
            cmd_opts.append("--graph")

        repo_git = scm.Scm(mir_root, scm_executable="git")
        output_str = repo_git.log(cmd_opts, with_stdout=with_stdout)
        if output_str:
            logging.info("\n%s" % output_str)

        return code.MirCode.RC_OK

    def run(self):
        logging.debug("command log: %s", self.args)

        return CmdLog.run_with_args(mir_root=self.args.mir_root,
                                    decorate=self.args.decorate,
                                    oneline=self.args.oneline,
                                    graph=self.args.graph,
                                    dog=self.args.dog)


def bind_to_subparsers(subparsers, parent_parser):  # pragma: no cover
    log_arg_parser = subparsers.add_parser("log",
                                           parents=[parent_parser],
                                           description="use this command to show mir repo log",
                                           help="show mir repo log")
    log_arg_parser.add_argument("--oneline", help="print log in one line", action="store_true")
    log_arg_parser.add_argument("--decorate", help="print log in a pretty way", action="store_true")
    log_arg_parser.add_argument("--graph", help="print log in a graphic way", action="store_true")
    log_arg_parser.add_argument("--dog", help="print log in one line graphic pretty way", action="store_true")
    log_arg_parser.set_defaults(func=CmdLog)



import logging
import os

from mir import scm
from mir.commands import base
from mir.tools import checker, code


class CmdCommit(base.BaseCommand):
    @staticmethod
    def run_with_args(mir_root: str, msg: str):
        return_code = checker.check(mir_root,
                                    [checker.Prerequisites.IS_INSIDE_MIR_REPO, checker.Prerequisites.IS_DIRTY])
        if return_code != code.MirCode.RC_OK:
            return return_code

        repo_git = scm.Scm(root_dir=mir_root, scm_executable='git')
        repo_dvc = scm.Scm(root_dir=mir_root, scm_executable='dvc')

        for f in ["metadatas.mir", "annotations.mir", "keywords.mir", "tasks.mir"]:
            if os.path.isfile(os.path.join(mir_root, f)):
                repo_dvc.add(f)
        repo_git.add(mir_root)

        output_str = repo_git.commit(["-m", msg])
        logging.info("\n%s" % output_str)

        return code.MirCode.RC_OK

    def run(self):
        logging.debug("command commit: %s", self.args)

        return CmdCommit.run_with_args(mir_root=self.args.mir_root, msg=self.args.cmt_msg)


def bind_to_subparsers(subparsers, parent_parser):  # pragma: no cover
    commit_arg_parser = subparsers.add_parser("commit",
                                              parents=[parent_parser],
                                              description="use this command to commit changes",
                                              help="commit changes")
    commit_arg_parser.add_argument("-m", dest="cmt_msg", type=str, help="message of this commit")
    commit_arg_parser.set_defaults(func=CmdCommit)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from ymir_proto.gen import mir_controller_service_pb2 as mir__controller__service__pb2


class mir_controller_serviceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.data_manage_request = channel.unary_unary(
                '/ymir.controller.mir_controller_service/data_manage_request',
                request_serializer=mir__controller__service__pb2.GeneralReq.SerializeToString,
                response_deserializer=mir__controller__service__pb2.GeneralResp.FromString,
                )


class mir_controller_serviceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def data_manage_request(self, request, context):
        """
        APIS FOR DATA MANAGEMENT
        KEY CONCEPTS
        sandbox: sandbox = sandbox_dir + docker_container
        sandbox_dir = sandbox_root + user_name
        docker_container = container of docker_image
        where sandbox_root and docker_image are get from cli args
        one user should have only one sandbox
        but can have multiple mir repos in this sandbox

        CREATE_SANDBOX
        creates a sandbox for a single user
        Args:
        GeneralReq.user: user name for this sandbox
        Returns:
        0: success
        errors when:
        sandbox already exists
        other system errors occurred

        REMOVE_SANDBOX
        removes a sandbox for a single user
        it also removes all contents in the sandbox
        Args:
        GeneralReq.user: user name for this sandbox
        Returns:
        0: success
        errors when:
        sandbox not exists
        other system errors occurred

        START_SANDBOX
        starts a sandbox for a single user
        Args:
        GeneralReq.user: user name for this sandbox
        Returns:
        0: success
        errors when:
        sandbox not exists
        sandbox already started
        other docker errors occurred

        STOP_SANDBOX
        stops a sandbox for a single user
        Args:
        GeneralReq.user: user name for this sandbox
        Returns:
        0: success
        errors when:
        sandbox not exists
        sandbox already stopped
        other docker errors occurred

        INIT
        init a new mir repo in a running sandbox
        Args:
        GeneralReq.user: user name for this sandbox
        GeneralReq.repo: repo name
        Returns:
        0: success
        errors when:
        sandbox not exists
        sandbox not running
        already have mir repo with the same name
        other docker errors occurred
        other mir errors occurred

        BRANCH_LIST
        list all branches in running sandbox for user
        Args:
        GeneralReq.user: user name for this sandbox
        GeneralReq.repo: repo name
        GeneralReq.ext_bool: if true, lists remote branches
        if false, lists local branches
        Returns:
        0: success
        ext_strs: branches
        errors when:
        sandbox not exists
        sandbox not running
        repo not found
        other docker errors occurred
        other mir errors occurred

        BRANCH_DEL
        remove one branch in running sandbox for user
        Args:
        GeneralReq.user: user name for this sandbox
        GeneralReq.repo: repo name
        GeneralReq.ext_str: branch to be deleted
        GeneralReq.ext_bool: force delete even if this branch has not been merged yet
        Returns:
        0: success
        errors when:
        sandbox not exists
        sandbox not running
        repo not found
        branch not found
        branch not merged if ext_bool is false
        other docker errors occurred
        other mir errors occurred

        CHECKOUT_COMMIT
        checkout to another commit, or to another branch, or to another tag
        Args:
        GeneralReq.user: user name for this sandbox
        GeneralReq.repo: repo name
        GeneralReq.ext_str: branch name, tag name or commit id
        Returns:
        0: success
        errors when:
        sandbox not exists
        sandbox not running
        repo not found
        branch, tag or commit not found
        other docker errors occurred
        other mir errors occurred

        CHECKOUT_BRANCH
        create a new branch in a running sandbox for user
        Args:
        GeneralReq.user: user name for this sandbox
        GeneralReq.repo: repo name
        GeneralReq.ext_str: new branch name
        Returns:
        0: success
        errors when:
        sandbox not exists
        sandbox not running
        repo not found
        new branch name already exists
        other docker errors occurred
        other mir errors occurred

        CLONE
        clones a mir repo from server
        Args:
        GeneralReq.user: user name for this sandbox
        GeneralReq.repo: repo url
        Returns:
        0: success
        errors when:
        sandbox not exists
        sandbox not running
        repo url not available
        other docker errors occurred
        other mir errors occurred

        COMMIT
        commit changes for mir repo
        Args:
        GeneralReq.user: user name for this sandbox
        GeneralReq.repo: repo name
        GeneralReq.ext_str: commit messages, multi lines enabled
        Returns:
        0: success
        errors when:
        sandbox not exists
        sandbox not running
        repo not found
        commit messages empty, or contains only spaces, tabs or line breaks
        other docker errors occurred
        other mir errors occurred

        FILTER
        filter assets (currently by asset keywords) in mir repo
        Args:
        GeneralReq.user: user name for this sandbox
        GeneralReq.repo: repo name
        GeneralReq.ext_str: predicates, keywords separated by commas or semicolons
        comma means AND
        semicolon means OR
        for example: `person; cat, dog` means to filter assets which
        have person, or have both cat and dog in asset keywords list
        attention that comma (means AND) has higher priority
        Returns:
        0: success
        errors when:
        sandbox not exists
        sandbox not running
        repo not found
        predicate empty
        other docker errors occurred
        other mir errors occurred

        LOG
        get log from repo
        Args:
        GeneralReq.user: user name for this sandbox
        GeneralReq.repo: repo name
        Returns:
        0: success
        GeneralResp.ext_strs: log infos
        errors when:
        sandbox not exists
        sandbox not running
        repo not found
        other docker errors occurred
        other mir errors occurred

        MERGE
        merges current repo with another
        Args:
        GeneralReq.user: user name for this sandbox
        GeneralReq.repo: repo name
        GeneralReq.ext_int32: merge stragety, 0: MIX, 1: GUEST
        GeneralReq.ext_str: guest branch name
        Returns:
        0: success
        errors when:
        sandbox not exists
        sandbox not running
        repo not found
        other docker errors occurred
        other mir errors occurred

        PULL
        pulls (updates) contents from server
        Args:
        GeneralReq.user: user name for this sandbox
        GeneralReq.repo: repo name
        Returns:
        0: success
        errors when:
        sandbox not exists
        sandbox not running
        repo not found
        other docker errors occurred
        other mir errors occurred

        PUSH
        pushes local commits to server
        Args:
        GeneralReq.user: user name for this sandbox
        GeneralReq.repo: repo name
        GeneralReq.ext_bool: creates new branch on server
        Returns:
        0: success
        errors when:
        sandbox not exists
        sandbox not running
        repo not found
        other docker errors occurred
        other mir errors occurred

        RESET: currently not available

        STATUS
        shows status of current repo
        Args:
        GeneralReq.user: user name for this sandbox
        GeneralReq.repo: repo name
        Returns:
        0: success
        GeneralResp.message: summary of current repo
        errors when:
        sandbox not exists
        sandbox not running
        repo not found
        other docker errors occurred
        other mir errors occurred
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_mir_controller_serviceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'data_manage_request': grpc.unary_unary_rpc_method_handler(
                    servicer.data_manage_request,
                    request_deserializer=mir__controller__service__pb2.GeneralReq.FromString,
                    response_serializer=mir__controller__service__pb2.GeneralResp.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'ymir.controller.mir_controller_service', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class mir_controller_service(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def data_manage_request(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ymir.controller.mir_controller_service/data_manage_request',
            mir__controller__service__pb2.GeneralReq.SerializeToString,
            mir__controller__service__pb2.GeneralResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import enum
import logging
import os
import sys
from typing import List

from mir.tools import code, mir_repo_utils


@enum.unique
class Prerequisites(enum.IntEnum):
    NOTHING = 0
    IS_INSIDE_GIT_REPO = enum.auto()
    IS_OUTSIDE_GIT_REPO = enum.auto()
    IS_INSIDE_MIR_REPO = enum.auto()
    IS_OUTSIDE_MIR_REPO = enum.auto()
    IS_DIRTY = enum.auto()
    IS_CLEAN = enum.auto()


_DEFAULT_PREREQUISTITES = [Prerequisites.IS_INSIDE_MIR_REPO, Prerequisites.IS_CLEAN]  # type: List[Prerequisites]


# check mir root
def check(mir_root: str, prerequisites: List[Prerequisites] = _DEFAULT_PREREQUISTITES) -> int:
    for item in prerequisites:
        checker_name = "_check_{}".format(item.name.lower())
        checker_func = getattr(sys.modules[__name__], checker_name)
        return_code = checker_func(mir_root)
        if return_code != code.MirCode.RC_OK:
            logging.info("check failed: {}".format(item.name.lower()))
            return return_code
    return code.MirCode.RC_OK


def _check_nothing(mir_root: str) -> int:
    return code.MirCode.RC_OK


def _check_is_inside_git_repo(mir_root: str) -> int:
    return (code.MirCode.RC_OK if os.path.isdir(os.path.join(mir_root, ".git")) else code.MirCode.RC_ERROR_INVALID_ARGS)


def _check_is_outside_git_repo(mir_root: str) -> int:
    return (code.MirCode.RC_OK
            if not os.path.isdir(os.path.join(mir_root, ".git")) else code.MirCode.RC_ERROR_INVALID_ARGS)


def _check_is_inside_mir_repo(mir_root: str) -> int:
    return (code.MirCode.RC_OK if os.path.isdir(os.path.join(mir_root, ".git"))
            and os.path.isdir(os.path.join(mir_root, ".dvc")) else code.MirCode.RC_ERROR_INVALID_MIR_REPO)


def _check_is_outside_mir_repo(mir_root: str) -> int:
    return (code.MirCode.RC_OK if not os.path.isdir(os.path.join(mir_root, ".git"))
            or not os.path.isdir(os.path.join(mir_root, ".dvc")) else code.MirCode.RC_ERROR_INVALID_ARGS)


def _check_is_dirty(mir_root: str) -> int:
    is_dirty = mir_repo_utils.mir_check_repo_dirty(mir_root)
    return code.MirCode.RC_OK if is_dirty else code.MirCode.RC_ERROR_INVALID_ARGS


def _check_is_clean(mir_root: str) -> int:
    is_dirty = mir_repo_utils.mir_check_repo_dirty(mir_root)
    return code.MirCode.RC_OK if not is_dirty else code.MirCode.RC_ERROR_DIRTY_REPO



import logging
import os

import google.protobuf.json_format as json_format

from mir.commands import base
from mir.tools import checker, code, mir_repo_utils, mir_storage
from ymir_proto.gen import mir_common_pb2 as mir_common
from ymir_proto.gen import mir_entities_pb2 as mirpb


class CmdStatus(base.BaseCommand):
    def run(self):
        logging.debug("command status: %s", self.args)

        return self.run_with_args(mir_root=self.args.mir_root,
                                  va=self.args.va,
                                  vm=self.args.vm,
                                  vk=self.args.vk,
                                  vt=self.args.vt)

    @staticmethod
    def run_with_args(mir_root, va=False, vm=False, vk=False, vt=False):
        return_code = checker.check(mir_root, [checker.Prerequisites.IS_INSIDE_MIR_REPO])
        if return_code != code.MirCode.RC_OK:
            return return_code

        metadata_file = os.path.join(mir_root, mir_storage.mir_path(mir_common.MirStorage.MIR_METADATAS))
        if os.path.isfile(metadata_file):
            with open(metadata_file, "rb") as file:
                mir_metadatas = mirpb.MirMetadatas()
                mir_metadatas.ParseFromString(file.read())
                CmdStatus._show_metadatas(mir_metadatas, vm)
        else:
            logging.warning("metadatas.mir not exist.")

        annotations_file = os.path.join(mir_root, mir_storage.mir_path(mir_common.MirStorage.MIR_ANNOTATIONS))
        if os.path.isfile(annotations_file):
            with open(annotations_file, "rb") as file:
                mir_annotations = mirpb.MirAnnotations()
                mir_annotations.ParseFromString(file.read())  # TODO: VERY SLOW IN ParseFromString
                CmdStatus._show_annotations(mir_annotations, va)
        else:
            logging.warning("annotations.mir not exist.")

        keywords_file = os.path.join(mir_root, mir_storage.mir_path(mir_common.MirStorage.MIR_KEYWORDS))
        if os.path.isfile(keywords_file):
            with open(keywords_file, "rb") as file:
                mir_keywords = mirpb.MirKeywords()
                mir_keywords.ParseFromString(file.read())
                CmdStatus._show_keywords(mir_keywords, vk)
        else:
            logging.warning("keywords.mir not exist.")

        tasks_file = os.path.join(mir_root, mir_storage.mir_path(mir_common.MirStorage.MIR_TASKS))
        if os.path.isfile(tasks_file):
            with open(tasks_file, "rb") as file:
                mir_tasks = mirpb.MirTasks()
                mir_tasks.ParseFromString(file.read())
                CmdStatus._show_tasks(mir_tasks, vt)
        else:
            logging.warning("tasks.mir not exist.")

        repo_dirty = mir_repo_utils.mir_check_repo_dirty(mir_root=mir_root)
        if repo_dirty:
            logging.info("repo: dirty")
        else:
            logging.info("repo: clean")

        return code.MirCode.RC_OK

    @staticmethod
    def _show_metadatas(mir_metadatas: mirpb.MirMetadatas, verbose: bool):
        logging.info("metadatas.mir: assets: %d", len(mir_metadatas.attributes))
        if verbose:
            logging.info(json_format.MessageToDict(mir_metadatas))

    @staticmethod
    def _show_annotations(mir_annotations: mirpb.MirAnnotations, verbose: bool):
        logging.info("annotations.mir: %d", len(mir_annotations.task_annotations))
        for task_id, task_annotations in mir_annotations.task_annotations.items():
            logging.info("  task: %s: images: %d",
                         task_id, len(task_annotations.image_annotations))
            if verbose:
                logging.info(json_format.MessageToDict(mir_annotations))

    @staticmethod
    def _show_keywords(mir_keywords: mirpb.MirKeywords, verbose: bool):
        logging.info("keywords.mir: assets: %d", len(mir_keywords.keywords))
        if verbose:
            logging.info(json_format.MessageToDict(mir_keywords))

    @staticmethod
    def _show_tasks(mir_tasks: mirpb.MirTasks, verbose: bool):
        logging.info("tasks.mir: tasks: %d", len(mir_tasks.tasks))
        if verbose:
            logging.info(json_format.MessageToDict(mir_tasks))


def bind_to_subparsers(subparsers, parent_parser):  # pragma: no cover
    status_arg_parser = subparsers.add_parser("status",
                                              parents=[parent_parser],
                                              description="use this command to show current workspace status",
                                              help="show current workspace status")
    status_arg_parser.add_argument("--vm", dest="vm", action="store_true", help="show metadatas details")
    status_arg_parser.add_argument("--va", dest="va", action="store_true", help="show annotations details")
    status_arg_parser.add_argument("--vk", dest="vk", action="store_true", help="show keywords details")
    status_arg_parser.add_argument("--vt", dest="vt", action="store_true", help="show tasks details")
    status_arg_parser.set_defaults(func=CmdStatus)



import datetime
import logging
from typing import Optional, Set

from mir.commands import base
from mir.tools import checker, code, mir_storage, mir_storage_ops, revs_parser
from ymir_proto.gen import mir_entities_pb2 as mirpb
from ymir_proto.gen import mir_common_pb2 as mir_common
from ymir_proto.util import ids


class CmdFilter(base.BaseCommand):
    # private: misc
    def __preds_set_from_str(preds_str: str, cls_mgr: ids.ClassIdManager) -> Set[int]:
        if not preds_str:
            return set()

        return set(cls_mgr.id_for_names(preds_str.split(";")))

    @staticmethod
    def __predefined_preds_match(asset_ids_set: Set[str], mir_keywords: mirpb.MirKeywords,
                                 preds_set: Set[int]) -> Set[str]:
        # if don't need preds filter, returns all
        if not preds_set:
            return asset_ids_set

        matched_asset_ids_set = set()  # type: Set[str]
        for asset_id in asset_ids_set:
            if asset_id not in mir_keywords.keywords:
                continue

            keyids_set = set(mir_keywords.keywords[asset_id].predifined_keyids)
            if not keyids_set:
                continue

            if keyids_set & preds_set:
                matched_asset_ids_set.add(asset_id)
        return matched_asset_ids_set

    @staticmethod
    def __predefined_excludes_match(asset_ids_set: Set[str], mir_keywords: mirpb.MirKeywords,
                                    excludes_set: Set[int]) -> Set[str]:
        # if don't need excludes filter, returns all
        if not excludes_set:
            return asset_ids_set

        matched_asset_ids_set = set()  # type: Set[str]
        for asset_id in asset_ids_set:
            if asset_id in mir_keywords.keywords:
                keyids_set = set(mir_keywords.keywords[asset_id].predifined_keyids)
                if keyids_set & excludes_set:
                    continue
            matched_asset_ids_set.add(asset_id)
        return matched_asset_ids_set

    # public: run cmd
    @staticmethod
    def run_with_args(mir_root: str, in_cis: Optional[str], ex_cis: Optional[str], src_revs: str, dst_rev: str) -> int:
        # check args
        if not in_cis and not ex_cis:
            logging.error("invalid args: no --in_cis (include class keys) and no --ex-cis (exclude class keys)")
            return code.MirCode.RC_ERROR_INVALID_ARGS

        if not src_revs:
            logging.error("invalid args: empty --src-revs")
            return code.MirCode.RC_ERROR_INVALID_ARGS
        src_typ_rev_tid = revs_parser.parse_single_arg_rev(src_revs)
        if not src_typ_rev_tid.rev:
            logging.error(f"invalid args: no rev in --src-revs: {src_typ_rev_tid}")
            return code.MirCode.RC_ERROR_INVALID_ARGS

        if not dst_rev:
            logging.error("invalid args: --dst-rev")
            return code.MirCode.RC_ERROR_INVALID_ARGS
        dst_typ_rev_tid = revs_parser.parse_single_arg_rev(dst_rev)
        if not dst_typ_rev_tid.tid:
            logging.error(f"invalid args: no tid in --dst-rev: {dst_typ_rev_tid}")
            return code.MirCode.RC_ERROR_INVALID_ARGS
        if not dst_typ_rev_tid.rev:
            logging.error(f"invalid args: no rev in --dst-rev: {dst_typ_rev_tid}")
            return code.MirCode.RC_ERROR_INVALID_ARGS

        return_code = checker.check(mir_root)
        if return_code != code.MirCode.RC_OK:
            return return_code

        mir_contents = mir_storage_ops.MirStorageOps.load(mir_root=mir_root,
                                                          mir_branch=src_typ_rev_tid.rev,
                                                          mir_storages=mir_storage.get_all_mir_storage())
        mir_metadatas = mir_contents[mir_common.MirStorage.MIR_METADATAS]
        mir_annotations = mir_contents[mir_common.MirStorage.MIR_ANNOTATIONS]
        mir_keywords = mir_contents[mir_common.MirStorage.MIR_KEYWORDS]
        mir_tasks = mir_contents[mir_common.MirStorage.MIR_TASKS]
        task_id = dst_typ_rev_tid.tid
        base_task_id = mir_annotations.head_task_id

        if task_id in mir_tasks.tasks:
            logging.error(f"invalid args: task id already exists: {task_id}")
            return code.MirCode.RC_ERROR_INVALID_ARGS
        if not base_task_id:
            raise ValueError("no base task id in tasks.mir")

        assert len(mir_annotations.task_annotations.keys()) == 1
        base_task_annotations = mir_annotations.task_annotations[base_task_id]  # type: mirpb.SingleTaskAnnotations

        class_manager = ids.ClassIdManager()
        preds_set = CmdFilter.__preds_set_from_str(in_cis, class_manager)  # type: Set[int]
        excludes_set = CmdFilter.__preds_set_from_str(ex_cis, class_manager)  # type: Set[int]

        asset_ids_set = set(mir_metadatas.attributes.keys())
        logging.info(f"assets count: {len(asset_ids_set)}")
        asset_ids_set = CmdFilter.__predefined_preds_match(asset_ids_set, mir_keywords, preds_set)
        if preds_set:
            logging.info(f"assets count after select: {len(asset_ids_set)}")
        asset_ids_set = CmdFilter.__predefined_excludes_match(asset_ids_set, mir_keywords, excludes_set)
        if excludes_set:
            logging.info(f"assets count after exclude: {len(asset_ids_set)}")

        if not asset_ids_set:
            logging.info("matched nothing with pred: {}, excludes: {}, please try another preds".format(in_cis, ex_cis))
            return code.MirCode.RC_ERROR_INVALID_ARGS

        matched_mir_metadatas = mirpb.MirMetadatas()
        matched_mir_annotations = mirpb.MirAnnotations()
        matched_mir_keywords = mirpb.MirKeywords()

        # generate matched metadatas, annotations and keywords
        for asset_id in asset_ids_set:
            # generate `matched_mir_metadatas`
            asset_attr = mir_metadatas.attributes[asset_id]
            matched_mir_metadatas.attributes[asset_id].CopyFrom(asset_attr)

        joint_ids = asset_ids_set & mir_keywords.keywords.keys()
        for asset_id in joint_ids:
            # generate `matched_mir_keywords`
            matched_mir_keywords.keywords[asset_id].CopyFrom(mir_keywords.keywords[asset_id])

        # generate `matched_mir_annotations`
        joint_ids = asset_ids_set & base_task_annotations.image_annotations.keys()
        for asset_id in joint_ids:
            matched_mir_annotations.task_annotations[task_id].image_annotations[asset_id].CopyFrom(
                base_task_annotations.image_annotations[asset_id])

        # generate matched tasks
        task = mirpb.Task()
        task.type = mir_common.TaskType.TaskTypeFilter
        task.name = f"filter bid: {base_task_id}, tid: {task_id}, select: {in_cis} exclude: {ex_cis}"
        task.base_task_id = base_task_id
        task.task_id = task_id
        task.timestamp = int(datetime.datetime.now().timestamp())
        mir_storage_ops.add_mir_task(mir_tasks, task)

        logging.info("matched: %d, overriding current mir repo", len(matched_mir_metadatas.attributes))

        matched_mir_contents = {
            mir_common.MirStorage.MIR_METADATAS: matched_mir_metadatas,
            mir_common.MirStorage.MIR_ANNOTATIONS: matched_mir_annotations,
            mir_common.MirStorage.MIR_KEYWORDS: matched_mir_keywords,
            mir_common.MirStorage.MIR_TASKS: mir_tasks,
        }

        mir_storage_ops.MirStorageOps.save_and_commit(mir_root=mir_root,
                                                      mir_branch=dst_typ_rev_tid.rev,
                                                      his_branch=src_typ_rev_tid.rev,
                                                      mir_datas=matched_mir_contents,
                                                      commit_message=task.name)

        return code.MirCode.RC_OK

    def run(self):
        logging.debug("command filter: %s", self.args)
        return CmdFilter.run_with_args(
            mir_root=self.args.mir_root,
            in_cis=self.args.in_cis,
            ex_cis=self.args.ex_cis,
            src_revs=self.args.src_revs,
            dst_rev=self.args.dst_rev)


def bind_to_subparsers(subparsers, parent_parser):  # pragma: no cover
    filter_arg_parser = subparsers.add_parser("filter",
                                              parents=[parent_parser],
                                              description="use this command to filter assets",
                                              help="filter assets")
    filter_arg_parser.add_argument("-p", dest="in_cis", type=str, help="type names")
    filter_arg_parser.add_argument("-P", dest="ex_cis", type=str, help="exclusive type names")
    filter_arg_parser.add_argument("--src-revs", dest="src_revs", type=str, help="type:rev@bid")
    filter_arg_parser.add_argument("--dst-rev", dest="dst_rev", type=str, help="rev@tid")
    filter_arg_parser.set_defaults(func=CmdFilter)



import imghdr
import logging
import os
import time
from PIL import Image, ImageFile
from typing import Optional, Tuple

from mir.tools.code import MirCode
from ymir_proto.gen import mir_entities_pb2 as mirpb
from ymir_proto.gen import mir_common_pb2 as mir_common


ImageFile.LOAD_TRUNCATED_IMAGES = True


def _generate_metadata_mir_pb(mir_metadatas: mirpb.MirMetadatas, dataset_name: str, sha1s: list,
                              hashed_asset_root: str) -> MirCode:
    """
    generate mirpb.MirMetadatas from sha1s
    """
    current_timestamp = int(time.time())  # this is a fake timestamp
    timestamp = mirpb.Timestamp()
    timestamp.start = current_timestamp
    timestamp.duration = 0  # image has no duraton

    for val in sha1s:
        metadata_attributes = mirpb.MetadataAttributes()
        metadata_attributes.timestamp.CopyFrom(timestamp)
        metadata_attributes.dataset_name = dataset_name

        # read file
        # if any exception occured, exit without any handler
        hashed_asset_path = os.path.join(hashed_asset_root, val)
        metadata_attributes.asset_type = _type_for_asset(hashed_asset_path)
        whcd = _shape_for_image(hashed_asset_path)
        if whcd:
            metadata_attributes.width = whcd[0]
            metadata_attributes.height = whcd[1]
            metadata_attributes.image_channels = whcd[2]

        mir_metadatas.attributes[val].CopyFrom(metadata_attributes)
    return MirCode.RC_OK


_ASSET_TYPE_STR_TO_ENUM_MAPPING = {
    "jpeg": mir_common.AssetTypeImageJpeg,
    "jpg": mir_common.AssetTypeImageJpeg,
    "png": mir_common.AssetTypeImagePng,
}


def _type_for_asset(asset_path: str) -> int:
    if not asset_path:
        return 0

    asset_type_str = imghdr.what(asset_path)
    if asset_type_str in _ASSET_TYPE_STR_TO_ENUM_MAPPING:
        return _ASSET_TYPE_STR_TO_ENUM_MAPPING[asset_type_str]
    else:
        return mir_common.AssetTypeUnknown


def _shape_for_image(image_asset_path: str) -> Optional[Tuple[int, int, int]]:
    """
    width, height, channels for image asset
    """
    image = Image.open(image_asset_path)
    width, height = image.size
    channel = len(image.split())
    return (width, height, channel)


def import_metadatas(mir_metadatas: mirpb.MirMetadatas, dataset_name: str, in_sha1_path: str,
                     hashed_asset_root: str) -> MirCode:
    # if not enough args, abort
    if (not in_sha1_path or not dataset_name or not hashed_asset_root):
        logging.error("invalid input paths")
        return MirCode.RC_ERROR_INVALID_ARGS

    if mir_metadatas is None:
        # some errors occured, show error message
        logging.critical("mir file is invalid, exit")
        return MirCode.RC_ERROR_INVALID_MIR_REPO

    # read sha1
    sha1s = []
    with open(in_sha1_path, "r") as in_file:
        for line in in_file.readlines():
            if not line or not line.strip():
                continue
            line_components = line.strip().split()
            if not line_components[0]:
                continue
            sha1s.append(line_components[0])
    if not sha1s:
        logging.critical("no sha1s found, exit")
        return MirCode.RC_ERROR_INVALID_ARGS

    # generate mir_metadatas
    ret = _generate_metadata_mir_pb(mir_metadatas=mir_metadatas,
                                    dataset_name=dataset_name,
                                    sha1s=sha1s,
                                    hashed_asset_root=hashed_asset_root)
    return ret



from typing import List


class TypRevTid:
    __slots__ = ("typ", "rev", "tid")

    def __init__(self) -> None:
        self.typ = ''
        self.rev = ''
        self.tid = ''

    def __repr__(self) -> str:
        return f"(t: {self.typ}, r: {self.rev}, t: {self.tid})"

    def __eq__(self, o: object) -> bool:
        if isinstance(o, TypRevTid):
            return self.typ == o.typ and self.rev == o.rev and self.tid == o.tid
        elif isinstance(o, tuple):
            return self.typ == o[0] and self.rev == o[1] and self.tid == o[2]
        else:
            return False


# public: parse methods
def parse_arg_revs(src_revs: str) -> List[TypRevTid]:
    typ_rev_tids = []
    if not src_revs:
        return typ_rev_tids

    comments = src_revs.split(";")
    for single_src_rev in comments:
        single_src_rev = single_src_rev.strip()
        if not single_src_rev:
            continue
        typ_rev_tids.append(__parse_single_arg_rev(single_src_rev))
    return typ_rev_tids


def parse_single_arg_rev(src_rev: str) -> TypRevTid:
    if ";" in src_rev:
        raise ValueError(f"src_rev: {src_rev} is not single")

    return __parse_single_arg_rev(src_rev)


# private: parse methods
def __parse_single_arg_rev(src_rev: str) -> TypRevTid:
    typ_rev_tid = TypRevTid()

    if not src_rev:
        return typ_rev_tid

    # parse typ and rev_and_tid
    contents = src_rev.split(':')
    if len(contents) > 2:
        raise ValueError(f"invalid arg: {src_rev}")
    rev_and_tid = ''
    if len(contents) == 1:
        # rev, rev@tid or @tid
        rev_and_tid = contents[0]
    elif len(contents) == 2:
        # typ:rev, typ:rev@tid or :rev@tid
        typ_rev_tid.typ = contents[0]
        rev_and_tid = contents[1]

    # parse rev and tid
    contents = rev_and_tid.split('@')
    if len(contents) > 2:
        raise ValueError(f"invalid arg: {src_rev}")
    if len(contents) == 1:
        # rev
        typ_rev_tid.rev = contents[0]
    elif len(contents) == 2:
        # rev or rev@tid
        typ_rev_tid.rev = contents[0]
        typ_rev_tid.tid = contents[1]

    if typ_rev_tid.typ and typ_rev_tid.typ not in {"tr", "va", "te"}:
        raise ValueError(f"invalid typ in typ:rev@tid: {src_rev}")

    return typ_rev_tid



from setuptools import setup, find_packages

from mir import version

print(version.__version__)

# Module dependencies
requirements = []
with open('requirements.txt') as f:
    for line in f.read().splitlines():
        requirements.append(line)

setup(
    name='pymir',
    version=version.__version__,
    python_requires=">=3.7.8",
    author="Scalable AI Team, Intellifusion",
    author_email="ymir-team@intellif.com",
    description="mir: A data version control tool for Ymir",
    url="http://192.168.70.8/scalable-ai/ymir",
    packages=find_packages(exclude=["*tests*"]),
    install_requires=requirements,
    include_package_data=True,
    entry_points={"console_scripts": ["mir = mir.main:main"]},
)



import os
from typing import Dict, List, Optional, Tuple


def ids_file_abs() -> str:
    ids_file_name = 'type_id_names.csv'
    ids_file_src = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))),
                                os.path.join('ids', ids_file_name))
    if not os.path.isfile(ids_file_src):
        raise RuntimeError("cannot find ids file: ", ids_file_src)
    return ids_file_src


class ClassIdManagerError(BaseException):
    pass


class ClassIdManager(object):
    """
    loads and manages type_name_ids mapping dictionary from csv file
    Discussions:
        csv file has the following format:
            type id, main type name, alias type name 1, alias type name 2, ...
        for example:
            27, airplane, aeroplane \n
            30, tv, tvmonitor \n
            31, dog \n
        for type id 27, we call airplane as the main type name, while aeroplane as alias. \n
        in our returned dict, this line will have 2 records:
            record 1: key: airplane, value: (27, None)
            record 2: key: aeroplane, value: (27, airplane)
        for each alias or main type name, you can get type idx and main type name from the value pair
    """
    __slots__ = ("_csv_path", "_type_name_id_dict", "_type_id_name_dict", "_max_id", "_dirty_id_names_dict")

    # life cycle
    def __init__(self) -> None:
        super().__init__()
        self._csv_path = None
        self._type_name_id_dict = {}  # type: Dict[str, Tuple[int, Optional[str]]]
        self._type_id_name_dict = {}  # type: Dict[int, str]
        self._dirty_id_names_dict = {}  # type: Dict[int, str]
        self._max_id = 0
        self.__load(ids_file_abs())

    # private: load and unload
    def __load(self, csv_path: str) -> bool:
        if not csv_path:
            raise ClassIdManagerError("empty path received")
        if self._csv_path:
            raise ClassIdManagerError("already loaded from: {}".format(self._csv_path))

        with open(csv_path, "r") as file:
            lines = file.readlines()
            for line in lines:
                components = line.split(",")
                if len(components) <= 1:
                    continue  # empty lines also ignored here

                type_id = int(components[0])
                if type_id > self._max_id:
                    self._max_id = type_id

                main_type_name = None

                for type_name_idx, type_name in enumerate(components[1:]):
                    type_name = type_name.strip()

                    # handle error situations
                    if type_name in self._type_name_id_dict:
                        previous_pair = self._type_name_id_dict[type_name]
                        raise ClassIdManagerError("dumplicate type name, previous: {}, now: {}".format(
                            previous_pair[0], type_id))
                    if not type_name:
                        raise ClassIdManagerError("empty type name for type id: {}".format(type_id))

                    if type_name_idx == 0:
                        main_type_name = type_name
                        self._type_name_id_dict[type_name] = (type_id, None)
                        self._type_id_name_dict[type_id] = type_name
                    else:
                        self._type_name_id_dict[type_name] = (type_id, main_type_name)

        self._csv_path = csv_path
        return True

    # public: general
    def save_dirty_dict(self) -> bool:
        if not self._csv_path:
            raise ClassIdManagerError("not loadef")
        if len(self._dirty_id_names_dict) == 0:
            return True  # no need to save

        with open(self._csv_path + ".new", "a") as f:
            for type_id, type_name in self._dirty_id_names_dict.items():
                f.write(f"{type_id}, {type_name}\n")
        return True

    def id_and_main_name_for_name(self, name: str, add_if_no_exists: bool = True) -> Tuple[int, Optional[str]]:
        name = name.strip()
        if not self._csv_path:
            raise ClassIdManagerError("not loadef")
        if not name:
            raise ClassIdManagerError("empty name")

        if name not in self._type_name_id_dict:
            if add_if_no_exists:
                self._max_id += 1
                self._type_name_id_dict[name] = (self._max_id, None)
                self._type_id_name_dict[self._max_id] = name
                self._dirty_id_names_dict[self._max_id] = name
            else:
                raise ClassIdManagerError("not exists: {}".format(name))

        return self._type_name_id_dict[name]

    def main_name_for_id(self, type_id: int) -> Optional[str]:
        return self._type_id_name_dict.get(type_id, None)

    def id_for_names(self, names: List[str]) -> List[int]:
        type_ids = []
        for name in names:
            type_ids.append(self.id_and_main_name_for_name(name=name, add_if_no_exists=False)[0])
        return type_ids

    def all_main_names(self) -> List[str]:
        return list(self._type_id_name_dict.values())

    def size(self) -> int:
        return len(self._type_name_id_dict)



import logging

from mir.tools import checker, data_exporter, revs_parser, mir_storage_ops
from mir.commands import base
from ymir_proto.gen import mir_common_pb2 as mir_common


class CmdExport(base.BaseCommand):
    def run(self):
        logging.debug(f"command export: {self.args}")

        return CmdExport.run_with_args(mir_root=self.args.mir_root,
                                       asset_dir=self.args.asset_dir,
                                       annotation_dir=self.args.annotation_dir,
                                       media_location=self.args.media_location,
                                       src_revs=self.args.src_revs,
                                       format=self.args.format)

    @staticmethod
    def run_with_args(mir_root: str, asset_dir: str, annotation_dir: str, media_location: str, src_revs: str,
                      format: str):
        # check args
        if not mir_root:
            mir_root = '.'
        if not format:
            format = 'ark'

        if not asset_dir or not annotation_dir:
            logging.error('empty --asset-dir or --annotation-dir')
            return mir_common.RC_CMD_INVALID_ARGS
        if not media_location:
            logging.error('empty --media-location')
            return mir_common.RC_CMD_INVALID_ARGS
        if not src_revs:
            logging.error('empty --src-revs')
            return mir_common.RC_CMD_INVALID_ARGS
        if format and (format not in data_exporter.SUPPORTED_EXPORT_FORMATS):
            logging.error(f"invalid --format: {format}")
            return mir_common.RC_CMD_INVALID_ARGS

        src_typ_rev_tid = revs_parser.parse_single_arg_rev(src_revs)
        if not src_typ_rev_tid.rev:
            logging.error(f"invalid --src-revs: {src_revs}: no rev found")
            return mir_common.RC_CMD_INVALID_ARGS

        return_code = checker.check(mir_root, prerequisites=[checker.Prerequisites.IS_INSIDE_MIR_REPO])
        if return_code != mir_common.RC_OK:
            return return_code

        format_type = data_exporter.format_type_from_str(format)

        # asset ids
        mir_metadatas = mir_storage_ops.MirStorageOps.load_single(mir_root=mir_root,
                                                                  mir_branch=src_typ_rev_tid.rev,
                                                                  ms=mir_common.MIR_METADATAS)
        asset_ids = set(mir_metadatas.attributes.keys())
        if not asset_ids:
            logging.error('nothing to export')
            return mir_common.RC_CMD_INVALID_ARGS

        # export
        data_exporter.export(mir_root=mir_root,
                             assets_location=media_location,
                             class_type_ids=None,
                             asset_ids=asset_ids,
                             asset_dir=asset_dir,
                             annotation_dir=annotation_dir,
                             need_suffix=True,
                             base_branch=src_typ_rev_tid.rev,
                             format_type=format_type)

        return mir_common.RC_OK


def bind_to_subparsers(subparsers, parent_parser):  # pragma: no cover
    exporting_arg_parser = subparsers.add_parser('export',
                                                 parents=[parent_parser],
                                                 description='use this command to export data',
                                                 help='export data')
    exporting_arg_parser.add_argument("--asset-dir",
                                      required=True,
                                      dest="asset_dir",
                                      type=str,
                                      help="export directory for assets")
    exporting_arg_parser.add_argument("--annotation-dir",
                                      required=True,
                                      dest="annotation_dir",
                                      type=str,
                                      help="export directory for annotations")
    exporting_arg_parser.add_argument('--media-location',
                                      dest='media_location',
                                      type=str,
                                      help='location of hashed assets')
    exporting_arg_parser.add_argument('--src-revs',
                                      dest='src_revs',
                                      type=str,
                                      help='rev@bid: source rev and base task id')
    exporting_arg_parser.add_argument('--format',
                                      dest='format',
                                      type=str,
                                      default="ark",
                                      choices=["ark", "voc", "none"],
                                      help='annotation format: ark / voc / none')
    exporting_arg_parser.set_defaults(func=CmdExport)



import imghdr
import logging
import os
import pathlib
import requests
import shutil
from typing import Dict, List, Optional, Union

from mir import scm


# project
def project_root() -> str:
    root = str(pathlib.Path(__file__).parent.parent.parent.absolute())
    return root


# mir repo infos
def mir_repo_head_name(git: Union[str, scm.CmdScm]) -> Optional[str]:
    """ get current mir repo head name (may be branch, or commit id) """
    git_scm = None
    if isinstance(git, str):
        git_scm = scm.Scm(git, scm_executable="git")
    elif isinstance(git, scm.CmdScm):
        git_scm = git
    else:
        raise ValueError("invalid git: needs str or CmdScm")

    git_result = git_scm.rev_parse(["--abbrev-ref", "HEAD"])
    if isinstance(git_result, str):
        return git_result
    elif isinstance(git_result, bytes):
        return git_result.decode("utf-8")
    return str(git_result)


def mir_repo_commit_id(git: Union[str, scm.CmdScm], branch="HEAD") -> str:
    """ get mir repo branch's commit id """
    git_scm = None
    if isinstance(git, str):
        git_scm = scm.Scm(git, scm_executable="git")
    elif isinstance(git, scm.CmdScm):
        git_scm = git
    else:
        raise ValueError("invalid git: needs str or CmdScm")

    git_result = git_scm.rev_parse(branch)
    if isinstance(git_result, str):
        return git_result
    elif isinstance(git_result, bytes):
        return git_result.decode("utf-8")
    return str(git_result)


# Store assets in asset_ids to out_root/sub_folder,
# return relative path to the out_root, staring with sub_folder.
# Set overwrite to False to avoid overwriting.
def store_assets_to_dir(asset_ids: List[str],
                        out_root: str,
                        sub_folder: str,
                        asset_location=None,
                        overwrite=False,
                        create_prefix=True,
                        need_suffix=True) -> Dict[str, str]:
    """
    load assets in location and save them to destination local folder
    Args:
        asset_ids: a list of asset ids (asset hashes)
        out_root: the root of output path
        sub_folder: sub folder to the output path
        asset_location: server location prefix of assets, if set to none, try to read it from mir repo config
    """
    # if out_root exists, but not a folder, raise error
    if os.path.exists(out_root) and not os.path.isdir(out_root):
        raise ValueError("invalid out_root")
    os.makedirs(out_root, exist_ok=True)
    sub_dir_abs = os.path.join(out_root, sub_folder)
    os.makedirs(sub_dir_abs, exist_ok=True)

    assets_location = _get_assets_location(asset_ids, asset_location)

    unknown_format_count = 0
    asset_id_to_rel_paths = {}  # Dict[str, str]
    for asset_id in asset_ids:
        if create_prefix:
            suffix = asset_id[-2:]
            sub_sub_folder_abs = os.path.join(sub_dir_abs, suffix)
            os.makedirs(sub_sub_folder_abs, exist_ok=True)
            sub_sub_folder_rel = os.path.join(sub_folder, suffix)
        else:
            sub_sub_folder_abs = sub_dir_abs
            sub_sub_folder_rel = sub_folder

        file_format = imghdr.what(assets_location[asset_id])
        if not file_format:
            file_format = 'unknown'
            unknown_format_count += 1

        file_name = (f"{asset_id}.{file_format.lower()}" if need_suffix else asset_id)
        asset_path_abs = os.path.join(sub_sub_folder_abs, file_name)  # path started from out_root
        asset_path_rel = os.path.join(sub_sub_folder_rel, file_name)  # path started from sub_folder
        _store_asset_to_location(assets_location[asset_id], asset_path_abs, overwrite=overwrite)
        asset_id_to_rel_paths[asset_id] = asset_path_rel

    if unknown_format_count > 0:
        logging.warning(f"unknown format asset count: {unknown_format_count}")

    return asset_id_to_rel_paths


def _store_asset_to_location(src: str, dst: str, overwrite: bool = False) -> None:
    if not src or not dst:
        return
    os.makedirs(os.path.dirname(dst), exist_ok=True)
    if not overwrite and os.path.isfile(dst):
        return
    if src.startswith('http'):  # from http request
        response = requests.get(src)
        if len(response.content) > 0:
            with open(dst, "wb") as f:
                f.write(response.content)
    elif src.startswith('/'):  # from filesystem, require abs path.
        shutil.copyfile(src, dst)
    else:
        raise ValueError('Invalid src path')


def _get_assets_location(asset_ids: List[str], asset_location) -> Dict[str, str]:
    """
    get asset locations
    Args:
        asset_ids: a list of asset ids (asset hashes)
        asset_location: the server location of assets.
    Returns:
        a dict, key: asset id, value: asset location url
    Raises:
        Attribute exception if asset_location is not set, and can not be found in config file
    """

    # asset_location is a required field.
    # CMD layer should NOT aware where the asset is stored.
    if not asset_location:
        raise ValueError("asset_location is not set.")

    return {id: os.path.join(asset_location, id) for id in asset_ids}



import os

from google.protobuf import json_format
from google.protobuf import message as pb_message
from typing import List, Dict, Optional, Union

from mir.tools import code, exodus, mir_storage, mir_repo_utils
from mir.commands.checkout import CmdCheckout
from mir.commands.commit import CmdCommit
from ymir_proto.gen import mir_common_pb2 as mir_common
from ymir_proto.gen import mir_entities_pb2 as mirpb


class MirStorageOps():
    # protected: presave actions
    @classmethod
    def _laze_pre_save(cls, mir_data):
        pass

    @classmethod
    def _annotations_pre_save(cls, mir_data):
        build_annotations_head_task_id(mir_data)

    @classmethod
    def _keywords_pre_save(cls, mir_data):
        build_keywords_index(mir_data)

    @classmethod
    def mir_pre_save(cls, ms: mir_common.MirStorage):
        MIR_PRE_SAVE = {
            mir_common.MirStorage.MIR_METADATAS: cls._laze_pre_save,
            mir_common.MirStorage.MIR_ANNOTATIONS: cls._annotations_pre_save,
            mir_common.MirStorage.MIR_KEYWORDS: cls._keywords_pre_save,
            mir_common.MirStorage.MIR_TASKS: cls._laze_pre_save,
        }
        return MIR_PRE_SAVE[ms]

    # public: save and load
    @classmethod
    def save(cls, mir_root, mir_datas: Dict[pb_message.Message, pb_message.Message]):
        for ms, mir_data in mir_datas.items():
            cls.mir_pre_save(ms)(mir_data)  # calc before save.

            mir_file_path = os.path.join(mir_root, mir_storage.mir_path(ms))
            with open(mir_file_path, "wb") as m_f:
                m_f.write(mir_data.SerializeToString())

    @classmethod
    def save_and_commit(cls, mir_root: str, mir_branch: str, his_branch: Optional[str],
                        mir_datas: Dict[pb_message.Message, pb_message.Message], commit_message: str) -> code.MirCode:
        """
        saves and commit all contents in mir_datas to branch: mir_branch;
        branch will be created if not exists, and it's history will be after his_branch

        Args:
            mir_root (str): path to mir repo
            mir_branch (str): the branch you wish to save to
            his_branch (Optional[str]): if mir_branch not exists, this is the branch where you wish to start with
            mir_datas (Dict[pb_message.Message, pb_message.Message]): datas you wish to save
            commit_message (str): commit messages

        Raises:
            ValueError: if commit message not provided

        Returns:
            code.MirCode: result code
        """
        if not mir_root:
            mir_root = '.'
        if not commit_message:
            raise ValueError("empty commit message")
        if not mir_branch:
            raise ValueError("empty mir branch")

        branch_exists = mir_repo_utils.mir_check_branch_exists(mir_root=mir_root, branch=mir_branch)
        if not branch_exists and not his_branch:
            raise ValueError(f"branch {mir_branch} not exists, and his_branch not provided")

        if branch_exists:
            his_branch = mir_branch
        return_code = CmdCheckout.run_with_args(mir_root=mir_root,
                                                commit_id=his_branch,
                                                branch_new=False)
        if return_code != code.MirCode.RC_OK:
            return return_code

        if not branch_exists:
            return_code = CmdCheckout.run_with_args(mir_root=mir_root,
                                                    commit_id=mir_branch,
                                                    branch_new=True)
            if return_code != code.MirCode.RC_OK:
                return return_code

        cls.save(mir_root=mir_root, mir_datas=mir_datas)

        return CmdCommit.run_with_args(mir_root=mir_root, msg=commit_message)

    @classmethod
    def load(cls,
             mir_root: str,
             mir_branch: str,
             mir_storages: List[pb_message.Message],
             as_dict: bool = False) -> Dict[pb_message.Message, Union[pb_message.Message, Dict]]:
        ret = {}
        for ms in mir_storages:
            ret[ms] = cls.load_single(mir_root=mir_root, mir_branch=mir_branch, ms=ms, as_dict=as_dict)
        return ret

    @classmethod
    def load_single(cls,
                    mir_root: str,
                    mir_branch: str,
                    ms: pb_message.Message,
                    as_dict: bool = False) -> Union[pb_message.Message, Dict]:
        mir_pb_type = mir_storage.mir_type(ms)
        mir_storage_data = mir_pb_type()
        with exodus.open_mir(mir_root=mir_root, file_name=mir_storage.mir_path(ms), rev=mir_branch, mode="rb") as f:
            mir_storage_data.ParseFromString(f.read())
        if as_dict:
            mir_storage_data = json_format.MessageToDict(mir_storage_data,
                                                         preserving_proto_field_name=True,
                                                         use_integers_for_enums=True)
        return mir_storage_data


# public: presave actions
def build_keywords_index(mir_keywords: mirpb.MirKeywords) -> code.MirCode:
    if not mir_keywords:
        raise RuntimeError("Invalid mir_keywords")

    mir_keywords.predifined_keyids_cnt.clear()
    mir_keywords.customized_keywords_cnt.clear()
    for asset_id, keywords in mir_keywords.keywords.items():
        for key_id in keywords.predifined_keyids:
            mir_keywords.predifined_keyids_cnt[key_id] += 1
            mir_keywords.index_predifined_keyids[key_id].asset_ids.append(asset_id)
        for keyword in keywords.customized_keywords:
            mir_keywords.customized_keywords_cnt[keyword] += 1

    mir_keywords.predifined_keyids_total = 0
    for cnt in mir_keywords.predifined_keyids_cnt.values():
        mir_keywords.predifined_keyids_total += cnt
    mir_keywords.customized_keywords_total = 0
    for cnt in mir_keywords.customized_keywords_cnt.values():
        mir_keywords.customized_keywords_total += cnt

    # Remove redundant index values.
    for key_id, assets in mir_keywords.index_predifined_keyids.items():
        mir_keywords.index_predifined_keyids[key_id].asset_ids[:] = set(
            mir_keywords.index_predifined_keyids[key_id].asset_ids)

    return code.MirCode.RC_OK


def build_annotations_head_task_id(mir_annotations: mirpb.MirAnnotations) -> code.MirCode:
    task_ids = list(mir_annotations.task_annotations.keys())
    if len(task_ids) == 1:
        mir_annotations.head_task_id = task_ids[0]
    elif len(task_ids) > 1:
        # * now we allows only one task id in each mir_annotations
        raise ValueError(f'more then one task ids found in mir_annotations: {task_ids}')


def add_mir_task(mir_tasks: mirpb.MirTasks, task: mirpb.Task):
    """
    add `task` to `mir_tasks`

    Args:
        mir_tasks (mirpb.MirTasks): contents in tasks.mir from repo
        task (mirpb.Task): new task you wish to add

    Raises:
        ValueError: if `task` has no task_id
        ValueError: if `task.task_id` already exists
    """
    if not task.task_id:
        raise ValueError("empty task id")
    if task.task_id in mir_tasks.tasks:
        raise ValueError(f"tasl id {task.task_id} already exists")

    orig_head_task_id = mir_tasks.head_task_id
    task.ancestor_task_id = orig_head_task_id
    mir_tasks.tasks[task.task_id].CopyFrom(task)
    mir_tasks.head_task_id = task.task_id



import logging

from mir import scm
from mir.commands import base
from mir.tools import checker, code


class CmdInit(base.BaseCommand):
    @staticmethod
    def run_with_args(mir_root):
        return_code = checker.check(
            mir_root, [checker.Prerequisites.IS_OUTSIDE_GIT_REPO, checker.Prerequisites.IS_OUTSIDE_MIR_REPO])
        if return_code != code.MirCode.RC_OK:
            return return_code

        repo_git = scm.Scm(root_dir=mir_root, scm_executable='git')
        repo_dvc = scm.Scm(root_dir=mir_root, scm_executable='dvc')
        repo_git.init()
        repo_dvc.init()

        repo_git.commit(["-m", "first commit"])

        return code.MirCode.RC_OK

    def run(self):
        logging.debug("command init: %s", self.args)

        return self.run_with_args(mir_root=self.args.mir_root)


def bind_to_subparsers(subparsers, parent_parser):  # pragma: no cover
    init_arg_parser = subparsers.add_parser("init",
                                            parents=[parent_parser],
                                            description="use this command to init mir repo",
                                            help="init mir repo")
    init_arg_parser.set_defaults(func=CmdInit)
